# -*- coding: utf-8 -*-

__authors__= "Juan Colino & Jorge Diez"
__email__= "juan.colino@gmail.com"
__license__= """
 --------------------------------------------------------------------------

 Waltz 0.1-beta3 - GTK Icon Management System
 Copyright (C) 2007 Juan Colino
 <juan.colino@gmail.com>

 --------------------------------------------------------------------------

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 --------------------------------------------------------------------------
"""


import pygtk
pygtk.require('2.0')
import gobject
import gtk
import gtk.gdk
import Image
import pango

import prefs

class MainWindow:
    "Clase que modela la ventana principal de la aplicacion"

    app = None
    prefs = None

    def __init__(self, app):
	"Crea la ventana con todos sus componentes"
	
	self.prefs = prefs.Prefs() # Patron Borg
	self.app = app

	# creamos window
	self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	self.window.set_icon(gtk.gdk.pixbuf_new_from_file('waltz.svg'))
	self.window.set_border_width(0)
	self.window.set_title("Waltz")
	self.window.set_position(gtk.WIN_POS_CENTER)
	self.window.set_default_size(self.prefs.prefs_dict['window width'],
	    self.prefs.prefs_dict['window height'])

	#  ########################################################################
	#  |                        gtk.VBox (vbox1)                              |
	#  |                                                                      |
	#  |  ##################################################################  |
	#  |  |     gtk.MenuBar (menubar)                                      |  |
	#  |  ##################################################################  |
	#  |  ##################################################################  |
	#  |  |     gtk.Toolbar (toolbar)                                      |  |
	#  |  ##################################################################  |
	#  |                                                                      |
	#  |  ##################################################################  |
	#  |  |     gtk.HPaned (hpaned)                                        |  |
	#  |  |                                                                |  |
	#  |  |  #######################   ############################        |  |
	#  |  |  |  gtk.VBox (vbox2)   |   |   gtk.Scrolled_window    |        |  |
	#  |  |  |                     |   |##########################|        |  |
	#  |  |  |  gtk.Label (label1) |   ||                        ||        |  |
	#  |  |  |  gtk.Label (label2) |   || gtk.TreeView (treeview)||        |  |
	#  |  |  |  gtk.Label (label3) |   ||                        ||        |  |
	#  |  |  |                     |   |##########################|        |  |
	#  |  |  #######################   ############################        |  |
	#  |  |                                                                |  |
	#  |  ##################################################################  |
	#  |                                                                      |
	#  |  ##################################################################  |
	#  |  |     gtk.Statusbar (satusbar)                                   |  |
	#  |  ##################################################################  |
	#  |                                                                      |
	#  ########################################################################

	# creamos vbox1
	self.vbox1 = gtk.VBox()

	# creamos la barra de menu y la barra de herramientas
	ui_description = """<ui>
	  <menubar name='Menubar'>
	    <menu action='FileMenu'>
	      <menuitem action='OpenDir'/>
	      <menuitem action='OpenTheme'/>
	      <separator/>
	      <menuitem action='Quit'/>
	    </menu>
	    <menu action='EditMenu'>
	      <menuitem action='Preferences'/>
	    </menu>
	    <menu action='HelpMenu'>
	      <menuitem action='About'/>
	    </menu>
	  </menubar>
	  <toolbar name='Toolbar'>
	    <toolitem action='OpenDir'/>
	    <separator/>
	    <toolitem action='Quit'/>
	  </toolbar>
	  <popup name='PopUpMenu'>
	    <menuitem action='OpenWithBitmap'/>
	    <menuitem action='OpenWithVectorial'/>
	  </popup>
	</ui>"""

	self.action_group = gtk.ActionGroup('WindowActions')
	self.actions = [
            ('FileMenu', None, '_File'),
            ('New',      gtk.STOCK_NEW, '_New', '<control>N',
             'Create a new file', self.create_about_dialog),
            ('OpenDir',     gtk.STOCK_OPEN, '_Open Directory', '<control>O',
             'Open a directory', self.create_opendir_dialog),
	    ('OpenTheme', gtk.STOCK_OPEN, 'Open _Theme', '<control>T',
	     'Open a theme', self.create_opentheme_dialog),
            ('Quit',     gtk.STOCK_QUIT, '_Quit', '<control>Q',
             'Quit application', self.destroy),
	    ('EditMenu', None, '_Edit'),
	    ('Preferences', gtk.STOCK_PREFERENCES, '_Preferences', '<control>P',
	     'Open the preferences dialog', self.create_preferences_dialog),
            ('HelpMenu', None, '_Help'),
            ('About',    gtk.STOCK_ABOUT, '_About', None, 'About application',
             self.create_about_dialog),
	    ('OpenWithBitmap', None, 'Open _with ' + self.prefs.prefs_dict['bitmap-editor'], '<control>W',
	     'Open with', self.app.open_with_bitmap),
	    ('OpenWithVectorial', None, 'Open _with ' + self.prefs.prefs_dict['vectorial-editor'], '<control>W',
	     'Open with', self.app.open_with_vectorial),
            ]
	self.action_group.add_actions(self.actions)

	self.ui = gtk.UIManager()
	self.ui.insert_action_group(self.action_group, 0)
	self.ui.add_ui_from_string(ui_description)
	self.window.add_accel_group(self.ui.get_accel_group())

	self.vbox1.pack_start(self.ui.get_widget('/Menubar'), expand = False)
	self.vbox1.pack_start(self.ui.get_widget('/Toolbar'), expand = False)


	# creamos hpaned
	self.hpaned = gtk.HPaned()

	# creamos vbox2
	self.vbox2 = gtk.VBox()

	# creamos las etiquetas y las empaquetamos en vbox2
	self.label1 = gtk.Label("label1")
	self.label1.set_max_width_chars(30)
	self.label1.set_ellipsize(pango.ELLIPSIZE_END)
	self.label1.show()
	self.label2 = gtk.Label("label2")
	self.label2.show()
	self.label3 = gtk.Label("label3")
	self.label3.show()
	self.vbox2.pack_start(child = self.label1, expand = False, fill=False, padding = 5)
	self.vbox2.pack_start(child = self.label2, expand = False, fill=False, padding = 5)
	self.vbox2.pack_start(child = self.label3, expand = False, fill=False, padding = 5)
	self.vbox2.show()

	# empaquetamos vbox2 en hpaned

	self.hpaned.pack1(self.vbox2, resize = False, shrink = False)
	self.hpaned.set_position(100)


	# creamos la gtk.ListStore y lo usamos para crear el treeview
	self.liststore = gtk.ListStore(str, str, gtk.gdk.Pixbuf)
	self.treemodelsort = gtk.TreeModelSort(self.liststore)
	self.treeview = gtk.TreeView(self.treemodelsort)

	# FIXME: ajustar propiedades

#	self.treeview.set_rules_hint(True)
#	self.treeview.set_reorderable(False)
#	self.treeview.set_enable_search(False)
	self.treeview_selection = self.treeview.get_selection()
	self.treeview_text_cell1 = gtk.CellRendererText()
	self.treeview_text_cell2 = gtk.CellRendererText()
	self.treeview_pix_cell = gtk.CellRendererPixbuf()
	self.treeview_column1 = gtk.TreeViewColumn('Name')
	self.treeview_column2 = gtk.TreeViewColumn('Size')
	self.treeview_column3 = gtk.TreeViewColumn('Image')
	self.treeview_column1.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
	self.treeview_column2.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
	self.treeview_column3.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
	self.treeview_column1.set_expand(True)
	self.treeview_column2.set_expand(True)
	self.treeview_column3.set_expand(True)
	self.treeview_column1.set_sort_column_id(0)
	self.treeview_column2.set_sort_column_id(1)
	self.treeview_column1.pack_start(self.treeview_text_cell1, True)
	self.treeview_column2.pack_start(self.treeview_text_cell2, True)
	self.treeview_column3.pack_start(self.treeview_pix_cell, True)
	self.treeview_column1.set_attributes(self.treeview_text_cell1, markup=0)
	self.treeview_column2.set_attributes(self.treeview_text_cell2, markup=1)
	self.treeview_column3.set_attributes(self.treeview_pix_cell, pixbuf=2)
	self.treeview.append_column(self.treeview_column1)
	self.treeview.append_column(self.treeview_column2)
	self.treeview.append_column(self.treeview_column3)
#	self.treeview.set_fixed_height_mode(True)	
	self.treeview.show()

	# creamos la scrolled_window y empaquetamos en ella el treeview
	self.scrolled_window = gtk.ScrolledWindow()
	self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
	self.scrolled_window.show()
	self.scrolled_window.add(self.treeview)

	# empaquetamos la scrolled_window en hpaned
	self.hpaned.pack2(self.scrolled_window, resize = False, shrink = False)
	self.hpaned.show()


	# empaquetamos hpaned en vbox1
	self.vbox1.pack_start(self.hpaned)
	
	# creamos la barra de estado
	self.statusbar = gtk.Statusbar()
	self.statusbar.show()

	# empaquetamos la barra de estado
	self.vbox1.pack_start(child = self.statusbar, expand = False)
	self.vbox1.show()

	# agregamos vbox1 a la ventana principal
	self.window.add(self.vbox1)
	self.window.show()

	# conectamos las señales
	self.window.connect("delete_event", self.delete_event)
	self.window.connect("destroy", self.destroy)
	self.treeview.connect("button_press_event", self.on_button_press) 


    def on_button_press(self, widget, event):
	"Maneja el evento 'button_press_event'"

	# TODO: Better way: http://www.async.com.br/faq/pygtk/index.py?req=show&file=faq13.017.htp
	if event.button == 3:
	    self.ui.get_widget("/PopUpMenu/OpenWithBitmap").set_sensitive(False)
	    self.ui.get_widget("/PopUpMenu/OpenWithVectorial").set_sensitive(False)
	    if self.treeview_selection.count_selected_rows() != 0:
		model, iterator = self.treeview_selection.get_selected()
		icon_name = model.get_value(iterator, 0)
		if icon_name.split('.')[-1] == 'png':
		    self.ui.get_widget("/PopUpMenu/OpenWithBitmap").set_sensitive(True)
		if icon_name.split('.')[-1] == 'svg':
		    self.ui.get_widget("/PopUpMenu/OpenWithVectorial").set_sensitive(True)
		    self.ui.get_widget("/PopUpMenu/OpenWithBitmap").set_sensitive(True)
	    self.ui.get_widget("/PopUpMenu").popup(None, None, None, event.button, event.time)
	else:
	    self.ui.get_widget("/PopUpMenu").popdown()
	    
	    

    def show_icons(self, icon_list):
	"Muestra una lista de iconos en el treeview"

	self.liststore.clear()
	for icon in icon_list:
	    icon_path = icon.get_path_to_file()
	    icon_name = icon.get_icon_name()
	    pixbuf = gtk.gdk.pixbuf_new_from_file(icon_path) 
	    self.liststore.append([icon_name, icon.get_icon_size(), pixbuf])

    def show_status(self, status_string, context):
	"Muestra un estado (status_string) en la barra de estado en un contexto (context)"

	context_id = self.statusbar.get_context_id(context)
	self.statusbar.pop(context_id)
	message_id = self.statusbar.push(context_id, status_string)
	return message_id
	
    def set_window_title(self, title):
	"Cambia el titulo de la ventana por 'title'"

	self.window.set_title(title)

    def create_about_dialog(self, event):
	"Crea el cuadro de dialogo about"

#	self.about_dialog = gtk.AboutDialog()
#	self.about_dialog.set_name('Waltz')
#	self.about_dialog.set_version('0.1')
#	self.about_dialog.set_copyright('Juan Colino')
#	self.about_dialog.set_comments('Waltz is an icon management system designed to handle icon themes')
#	self.about_dialog.set_license(__license__)
#	self.about_dialog.set_website('http://csl-waltz-forja.rediris.es')
#	self.about_dialog.set_website_label('WebSite')
#	self.about_dialog.set_authors(['Juan Colino Barrigón <juan.colino@gmail.com>'])
#	icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size("waltz.svg", 150, 150)
#	self.about_dialog.set_logo(icon_pixbuf)
#	self.about_dialog.show()

	# ===========
	# About tab
	# ===========
	color1 = '#333333'
	color2 = '#793566'

	about_dialog = gtk.Dialog("About", self.window, 0, (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
	about_dialog.set_has_separator(False)
	about_dialog.set_resizable(False)
	box = gtk.VBox(False, 0)
	box.set_border_width(5)
	notebook = gtk.Notebook()
	about_dialog.vbox.pack_start(notebook, False, False, 0)
	icon_pixbuf = gtk.gdk.pixbuf_new_from_file_at_size("waltz.svg", 150, 150)
	icon_image = gtk.Image()
	icon_image.set_from_pixbuf(icon_pixbuf)
	box.pack_start(icon_image, False, False, 10)
	label = gtk.Label()
	# FIXME: limpieza
	label.set_markup(
            '<big><big><big><big><b><span foreground="' + color1 + '">Wal</span>' +
   	    '<span foreground="' + color2 + '">tz</span> <span foreground="' + color1 + '">' +
            self.app.version +
            '</span></b></big></big></big></big>\n' +
            "\n" +
            "Waltz is an icon management system designed to handle icon themes." +
            "\n" +
            "It can open icon themes from index.theme files." +
            "\n" +
            "Waltz is licensed under the GNU General Public License." + "\n" +
            "\n" +
	    "<small>Copyright © 2007 Juan Colino &amp; Jorge Diez\n\n" +
	    "juan.colino@gmail.com</small>\n" +
	    "<small>http://csl-waltz.forja.rediris.es</small>\n")
    	box.pack_start(label, True, True, 0)
	label.set_justify(gtk.JUSTIFY_CENTER)

	notebook.insert_page(box, gtk.Label("About"))
        
        # =======================================================
        # Credits tab.
        # =======================================================
	box = gtk.VBox(False, 5)
	box.set_border_width(5)

	for nice_person in (('Pontus Ekberg', 'code inspiration'), ('Colaborator name', 'task')):
	    label = gtk.Label()
	    label.set_markup('<b>' + nice_person[0] + ':</b>   ' +
	    nice_person[1])
	    box.pack_start(label, False, False, 0)
	    label.set_alignment(0, 0)

	notebook.insert_page(box, gtk.Label("Credits"))
	about_dialog.vbox.show_all()
	about_dialog.run()
	about_dialog.destroy()

    def create_opentheme_dialog(self, event):
	""
	pass

    def create_preferences_dialog(self, event):
	"Crea el cuadro de dialogo preferences y guarda las preferencias una vez introducidas"
	prefs_dialog = gtk.Dialog("Preferences", self.window, 0,
	    (gtk.STOCK_APPLY, gtk.RESPONSE_APPLY,
	     gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
	     gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
	prefs_dialog.set_has_separator(False)
	prefs_dialog.set_resizable(False)

	display_box = gtk.VBox(False, 0)
	display_box.set_border_width(5)
	window_size_label = gtk.Label()
	window_size_label.set_markup("<b>Window size</b>")
	window_size_label.set_alignment(0, 0.5)
	display_box.pack_start(window_size_label, False, False, 0)
	window_size_table = gtk.Table(2, 2, False)
	window_width_entry = gtk.Entry()
	window_width_entry.set_text(str(self.prefs.prefs_dict['window width']))
	window_size_table.attach(gtk.Label("Initial window width:"), 0, 1, 0, 1)
	window_size_table.attach(window_width_entry, 1, 2, 0, 1)
	window_height_entry = gtk.Entry()
	window_height_entry.set_text(str(self.prefs.prefs_dict['window height']))
	window_size_table.attach(gtk.Label("Initial window height:"), 0, 1, 1, 2)
	window_size_table.attach(window_height_entry, 1, 2, 1, 2)
	display_box.pack_start(window_size_table, False, False, 0)

	launchers_box = gtk.VBox(False, 0)
	launchers_box.set_border_width(5)
	launchers_label = gtk.Label()
	launchers_label.set_markup("<b>Extern Launchers</b>")
	launchers_label.set_alignment(0, 0.5)
	launchers_box.pack_start(launchers_label, False, False, 0)
	launchers_table = gtk.Table(4, 2, False)
	vectorial_editor_name_entry = gtk.Entry()
	vectorial_editor_name_entry.set_text(str(self.prefs.prefs_dict['vectorial-editor']))
	launchers_table.attach(gtk.Label("Vectorial editor name:"), 0, 1, 0, 1)
	launchers_table.attach(vectorial_editor_name_entry, 1, 2, 0, 1)
	vectorial_editor_cmd_entry = gtk.Entry()
	vectorial_editor_cmd_entry.set_text(str(self.prefs.prefs_dict['vectorial-editor-cmd']))
	launchers_table.attach(gtk.Label("Vectorial editor command:"), 0, 1, 1, 2)
	launchers_table.attach(vectorial_editor_cmd_entry, 1, 2, 1, 2)
	bitmap_editor_name_entry = gtk.Entry()
	bitmap_editor_name_entry.set_text(str(self.prefs.prefs_dict['bitmap-editor']))
	launchers_table.attach(gtk.Label("Bitmap editor name:"), 0, 1, 2, 3)
	launchers_table.attach(bitmap_editor_name_entry, 1, 2, 2, 3)
	bitmap_editor_cmd_entry = gtk.Entry()
	bitmap_editor_cmd_entry.set_text(str(self.prefs.prefs_dict['bitmap-editor-cmd']))
	launchers_table.attach(gtk.Label("Vectorial editor command:"), 0, 1, 3, 4)
	launchers_table.attach(bitmap_editor_cmd_entry, 1, 2, 3, 4)
	launchers_box.pack_start(launchers_table, False, False, 0)

	notebook = gtk.Notebook()
	notebook.insert_page(display_box, gtk.Label("Display"))
	notebook.insert_page(launchers_box, gtk.Label("Launchers"))
	prefs_dialog.vbox.pack_start(notebook, False, False, 0)

	prefs_dialog.vbox.show_all()
	while True:
	    response = prefs_dialog.run()
	    if response == gtk.RESPONSE_APPLY or response == gtk.RESPONSE_ACCEPT:
		self.prefs.prefs_dict['window width'] = window_width_entry.get_text()
		self.prefs.prefs_dict['window height'] = window_height_entry.get_text()
		self.prefs.prefs_dict['vectorial-editor'] = vectorial_editor_name_entry.get_text()
		self.prefs.prefs_dict['vectorial-editor-cmd'] = vectorial_editor_cmd_entry.get_text()
		action = self.action_group.get_action("OpenWithVectorial")
		action.props.label = "Open _with " + self.prefs.prefs_dict['vectorial-editor']
		self.prefs.prefs_dict['bitmap-editor'] = bitmap_editor_name_entry.get_text()
		self.prefs.prefs_dict['bitmap-editor-cmd'] = bitmap_editor_cmd_entry.get_text()
		action = self.action_group.get_action("OpenWithBitmap")
		action.props.label = "Open _with " + self.prefs.prefs_dict['bitmap-editor']
		if response == gtk.RESPONSE_ACCEPT:
		    break
	    elif response == gtk.RESPONSE_CANCEL:
		break
	prefs_dialog.destroy()

    def create_opendir_dialog(self, event):
	"Crea el cuadro de dialogo open"
	open_dialog = gtk.FileChooserDialog("Open directory", self.window, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
		(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))
	open_dialog.set_default_response(gtk.RESPONSE_OK)

	response = open_dialog.run()
	if response == gtk.RESPONSE_OK:
	    print open_dialog.get_filename() + ' selected...'
	    self.app.read_icons_from_directory(open_dialog.get_filename())
	elif response == gtk.RESPONSE_CANCEL:
	    pass
	open_dialog.destroy()
	
    def delete_event(self, widget, event, data=None):
	print "delete event occurred"
	return False

    def destroy(self, widget, data=None):
	gtk.main_quit()

